Attribute VB_Name = "modBetterSplit"
Option Explicit

Function SplitWithChar(InString As String, _
    Delimiter As String, _
    GroupChar As String, _
    Optional CompareMethod As VbCompareMethod = vbTextCompare, _
    Optional IngoreMultipleDelimiters As Boolean = True, _
    Optional DeleteGroupCharacters As Boolean = False, _
    Optional Escape As String = vbNullString) As String()
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SplitWithChar
' This function is a replacment for VBA's Split function. Split has two
' problems. First, you cannot treat several elements as a single indivisible
' element, such as keeping a quoted string together even if there is a
' delimiter within the quoted strig. Second, Split will create an empty
' element in the result array if there are two consequtive delimiters with
' no text between them. This function addresses these problems.
'
'   Input Parameters:
'   =======================================================================
'       InString        the string to split
'       Delimiter       the delimiter used to break apart InString
'       GroupChar       a character that is used to delimit elements
'                       that are not to be split, such as a (") character
'                       If the GroupChar is used, there should be an
'                       even number of GroupChar characters in InString.
'                       An odd number of GroupChar characters may cause
'                       InString to be split in an unexpected manner.
'       CompareMode     if vbTextCompare, upper and lower case differences
'                       are ignored. If present and vbBinaryCompare, differences
'                       in upper and lower case are not treated as the same.
'                       to prevent splitting quoted string.
'       IgnoreMultipleDelimiters        This indicates that consequtive
'                       delimiters are to be ignored, which eliminates empty
'                       elements in the result array.
'       DeleteGroupCharacters   This indictes that the grouping characters
'                       be deleted from the element they delimit. E.g.,
'                       If InString is
'                           aa "bb cc" dd
'                       the second element will be
'                           "bb cc"
'                       if DeleteGroupCharacters is False, or
'                            bb cc
'                       if DeleteGroupCharacters is True. The default is False.
'       Escape          a single character that when prefixed to Delimiter,
'                       will prevent the split from splitting on that
'                       instance of the delimiter. For example, if InSring
'                       is
'                           aa|bb\|cc
'                       and the Delimiter is '|', the normal action is to
'                       split to a result of
'                           aa  bb\     cc
'                       If Escape is set to '\', the result is
'                           aa  bb|cc
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim SS() As String
    Dim S As String
    Dim N As Long
    Dim Sep As String
    Dim InGroup As Boolean
    Dim Esc As String
    
    
    If Delimiter = vbNullString Then
            ReDim SS(0 To 0)
            SS(0) = InString
            SplitWithChar = SS
            Exit Function
    End If
    S = InString
    For N = 5 To 255
        If InStr(1, S, Chr(N), vbBinaryCompare) = 0 Then
            Sep = Chr(N)
            Exit For
        End If
    Next N
    For N = 5 To 255
        If StrComp(Chr(N), Sep, vbTextCompare) <> 0 Then
            If InStr(1, S, Chr(N), vbBinaryCompare) = 0 Then
                Esc = Chr(N)
                Exit For
            End If
        End If
    Next N
        
    
    N = InStr(1, S, String2(2, Delimiter), CompareMethod)
    Do Until N = 0
        S = Replace(S, String2(2, Delimiter), Delimiter)
        N = InStr(1, S, String2(2, Delimiter), CompareMethod)
    Loop
    
    
    N = InStr(1, S, Escape & Delimiter, vbBinaryCompare)
    Do Until N = 0
        S = Replace(S, Escape & Delimiter, Esc)
        N = InStr(1, S, Escape & Delimiter, vbBinaryCompare)
    Loop
    
    
    For N = 1 To Len(S) - Len(Delimiter)
        If StrComp(Mid(InString, N, Len(GroupChar)), GroupChar, vbBinaryCompare) = 0 Then
            InGroup = Not InGroup
        End If
        If StrComp(Mid(InString, N, Len(Delimiter)), Delimiter, CompareMethod) = 0 Then
            If InGroup Then
                S = Left(S, N - 1) & Sep & Mid(S, N + Len(Sep))
            End If
        End If
    Next N
    SS = Split(S, Delimiter, , CompareMethod)
    For N = LBound(SS) To UBound(SS)
        SS(N) = Replace(SS(N), Sep, Delimiter)
        If DeleteGroupCharacters = True Then
            If Len(GroupChar) > 0 Then
                SS(N) = Replace(SS(N), GroupChar, vbNullString)
            End If
        End If
        If Len(Escape) > 0 Then
            SS(N) = Replace(SS(N), Esc, Delimiter)
        End If
    Next N
    SplitWithChar = SS
End Function

Private Function String2(Number As Long, Sequence As String) As String
    Dim S As String
    Dim N As Long
    For N = 1 To Number
        S = S & Sequence
    Next N
    String2 = S
End Function

